<?php

namespace Bydedi\Avalara\Adapter;

use Bydedi\Avalara\Model\Address;
use Bydedi\Avalara\Model\Customer;
use Bydedi\Avalara\Model\Product;
use Bydedi\Avalara\Model\Shipping;
use Commande;

/**
 * Class CommandeAdapter
 * @package Bydedi\Avalara\Adapter
 */
class CommandeAdapter
{
    /**
     * @var Commande
     */
    protected $commande;

    /**
     * CommandeAdapter constructor.
     * @param Commande $commande
     */
    public function __construct(Commande $commande)
    {
        $this->commande = $commande;
    }

    /**
     * ////////////////////////////////////
     * GETTERS & SETTERS
     * ////////////////////////////////////
     */

    /**
     * @return Commande
     */
    public function getCommande()
    {
        return $this->commande;
    }

    /**
     * @param $commande
     * @return $this
     */
    public function setCommande(Commande $commande)
    {
        $this->commande = $commande;
        return $this;
    }

    /**
     * ////////////////////////////////////
     * PUBLIC METHODS
     * ////////////////////////////////////
     */

    /**
     * @return Customer
     */
    public function getCustomer()
    {
        $customer = new Customer();
        $customer->code = $this->commande->getClient()->client_id;

        return $customer;
    }

    /**
     * @return Address
     */
    public function getAddressShipping(){
        $commande = $this->getCommande();

        $address = new Address();
        $address
            ->setLine1($commande->__get('livraison_rue'))
            ->setLine2($commande->__get('livraison_rue2'))
            ->setCity($commande->__get('livraison_ville'))
            ->setCountry($commande->__get('livraison_pays_code'))
            ->setPostalCode($commande->__get('livraison_cp'));

        if(!empty($commande->__get('livraison_state_us_code'))){
            $address->setRegion($commande->__get('livraison_state_us_code'));
        }

        return $address;
    }

    /**
     * @return array
     */
    public function getProducts()
    {
        $result = array();
        foreach($this->getCommande()->getProduit() as $p){
            $product = new Product();
            $product
                ->setIdentifier($p['commande_produit_id'])
                ->setPriceHt($p['prix_ht'])
                ->setQuantity((float)$p['produit_quantite'])
                ->setTaxCode('P0000000');
            // TODO : TaxCode susceptible de changer
            $result[] = $product;
        }
        return $result;
    }

    /**
     * @return Shipping
     */
    public function getShipping(){
        $shipping = new Shipping();
        $shipping
            ->setPriceHt($this->getCommande()->__get('commande_port_ht'))
            ->setTaxCode('FR020100');
        return $shipping;
    }
}
