<?php

namespace Bydedi\Avalara;

use Avalara\AvaTaxClient;
use Avalara\CompanyInitializationModel;
use Avalara\CompanyModel;
use Avalara\TransactionAddressType;
use Avalara\TransactionBuilder;
use Bydedi\Avalara\Model\Address;
use Bydedi\Avalara\Model\Customer;
use Bydedi\Avalara\Model\Product;
use Bydedi\Avalara\Model\Shipping;

/**
 * Class AvaTax
 *
 * Sandbox :
 * Admin Console URL: https://admin-development.avalara.net
 * Developer Documentation: https://developer.avalara.com/
 * Username: m.bouquet@dediservices.com
 * Temporary Password: DB0101EE7E
 * Account ID: 2000211587
 * License Key: 3FBCBA2180B06F10C2B051289A4F1EFCC504A0DF
 *
 */
class AvaTax
{
    // const INVOICE = DocumentType::C_SALESINVOICE;
    // const ORDER = DocumentType::C_SALESORDER;

    const INVOICE = 'SalesInvoice';
    const ORDER = 'SalesOrder';

    /**
     * @var AvaTax|null
     */
    static private $instance = null;

    /**
     * @var AvaTaxClient
     */
    protected $client;

    /**
     * @var CompanyModel
     */
    protected $company;

    /**
     * @var string $companyCode
     */
    protected $companyCode;

    /**
     * AvaTax constructor.
     * @param $config
     * @throws \Exception
     */
    private function __construct($config)
    {
        $this->client = new AvaTaxClient('website', '1.0', 'website01', $config['avalara']['avatax']['mode']);

        /*if (isset($config['avalara']['avatax']['account_id'], $config['avalara']['avatax']['license_key'])) {
            $this->client->withLicenseKey(
                $config['avalara']['avatax']['account_id'],
                $config['avalara']['avatax']['license_key']
            );
        }*/

        if (isset($config['avalara']['avatax']['username'], $config['avalara']['avatax']['password'])) {
            $this->client->withSecurity(
                $config['avalara']['avatax']['username'],
                $config['avalara']['avatax']['password']
            );
        }

        if(isset($config['avalara']['avatax']['company_code'])){
            $this->setCompanyCode($config['avalara']['avatax']['company_code']);
        }else{
            throw new \Exception('Erreur configuration Avalara - company_code manquant');
        }

        $ping = $this->client->ping();

        if ($ping->authenticated != true) {
            throw new \Exception('Error client Avalara - ping not good');
        }

        /*if (isset($config['avalara']['avatax']['company'])) {
            $this->setCompany($config['avalara']['avatax']['company']);
        }*/
    }

    /**
     * Get instance of the singleton
     * @param array $config
     * @return AvaTax
     */
    public static function getInstance(array $config)
    {
        if (!(self::$instance instanceof self)) {
            self::$instance = new self($config);
        }
        return self::$instance;
    }

    /**
     * @param $company
     */
    public function setCompany($company)
    {
        // TODO : check si company doit exister dans le compte ?
        $model = new CompanyInitializationModel();
        $model->city = $company['city'];
        $model->companyCode = $company['id'];
        $model->country = $company['code_country'];     // US
        $model->email = $company['email'];
        $model->faxNumber = null;
        $model->firstName = $company['first_name'];
        $model->lastName = $company['last_name'];
        $model->line1 = $company['address'];
        $model->line2 = $company['address2'];
        $model->mobileNumber = $company['mobile_number'];
        $model->phoneNumber = $company['phone_number'];
        $model->postalCode = $company['postal_code'];
        $model->region = $company['region'];
        $model->taxpayerIdNumber = null;
        $model->name = $company['name'];
        $model->title = null;
        $this->company = $this->client->companyInitialize($model);
    }

    /**
     * @return string
     */
    public function getCompanyCode()
    {
        return $this->companyCode;
    }

    /**
     * @param $companyCode
     * @return $this
     */
    public function setCompanyCode($companyCode)
    {
        $this->companyCode = $companyCode;
        return $this;
    }

    /**
     * @param $type : self::INVOICE | self::ORDER
     * @param Customer $customer
     * @param Address $from
     * @param Address $to
     * @param $products
     * @param bool $commit
     *
     * @return array
     */
    public function getTax($type, Customer $customer, Address $from, Address $to, Shipping $shipping, $products, $commit = false)
    {
        // https://github.com/avadev/AvaTax-REST-V2-PHP-SDK/blob/master/tests/test.php

        // TODO : codeCompany
        $tb = new TransactionBuilder($this->client, $this->getCompanyCode(), $type, $customer->code);

        $t = $tb->withAddress(
            TransactionAddressType::C_SHIPFROM,
            $from->line1,
            $from->line2,
            $from->line3,
            $from->city,
            $from->region,
            $from->postalCode,
            $from->country
        )->withAddress(
            TransactionAddressType::C_SHIPTO,
            $to->line1,
            $to->line2,
            $to->line3,
            $to->city,
            $to->region,
            $to->postalCode,
            $to->country
        );

        $productIdentifiers = array();
        /**
         * @var int $key
         * @var Product $product
         */
        foreach ($products as $key => $product) {
            $t->withLine($product->getPriceHt() * $product->getQuantity(), $product->getQuantity(), $product->getTaxCode());
            $productIdentifiers[$key] = $product->getIdentifier();
        }

        // Ajout de la ligne pour la livraison
        $t->withLine($shipping->getPriceHt(), 1.0, $shipping->getTaxCode());

        if ($commit == true) {
            $t->withCommit();
        }

        // send transaction to Avalara
        $model = $t->create();

        $result = $this->getTaxRates($model, $productIdentifiers);

        return [
            'model' => $model,
            'taxRates' => $result
        ];
    }

    /**
     * @param $model
     * @param $productIdentifiers
     * @return array
     */
    public function getTaxRates($model, $productIdentifiers){
        $result = array();

        if(!empty($model->lines)){
            foreach($model->lines as $key => $line){
                $res = 0;
                if(!empty($line->details)){
                    foreach ($line->details as $detail){
                        $res += $detail->rate;
                    }
                }
                if($key == count($productIdentifiers)){
                    $identifier = 'shipping';
                }else{
                    $identifier = $productIdentifiers[$key];
                }
                $result[$key] = array(
                    'identifier' => $identifier,
                    'tax_rate' => $res
                );
            }
        }

        return $result;
    }
}
